#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/wait.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#define SERVER_PORT		9999
#define BUF_SIZE		1024
#define MAX_HOST_NAME		50
#define DEFAULT_PROTOCOL	0
#define SOCKET_PATH		"/tmp/.mySocket"

void main() {
	int serverFd,clientFd,serverLen,pid,operand1,operand2,optVal;
	struct sockaddr_un serverAddr;
	char dataBuffer[BUF_SIZE];

	/* unlink previous file if any */
	unlink(SOCKET_PATH);
	/* clear our address */
	memset(&serverAddr,0,sizeof(serverAddr));
	/* set socket path and family */
	serverAddr.sun_family=AF_UNIX;
	strcpy(serverAddr.sun_path,SOCKET_PATH);
	/* open a UNIX socket */
	if ((serverFd=socket(AF_UNIX,SOCK_STREAM,DEFAULT_PROTOCOL))==-1) {
		perror("socket");
		exit(1);
	}
	/* set socket options:SO_REUSEADDR */
	optVal=1;
	if (setsockopt(serverFd,SOL_SOCKET,SO_REUSEADDR,&optVal,sizeof(optVal))==-1) {
		perror("setsockopt");
		exit(1);
	}
	/* bind the socket to address */
	if (bind(serverFd,(struct sockaddr*) &serverAddr,sizeof(serverAddr))==-1) {
		perror("bind");
		exit(1);
	}
	/* set maximum pending connection requests */
	if (listen(serverFd,5)==-1) {
		perror("listen");
		exit(1);
	}
	serverLen=sizeof(serverAddr);
	/* now accetpt and serve */
	for (;(clientFd=accept(serverFd,(struct sockaddr*) &serverAddr,&serverLen));) {
		if (clientFd==-1) {
			perror("accept");
			exit(1);
		}
		/* now fork a new process to deal with the calculation */
		if ((pid=fork())==-1) {
			perror("fork");
			close(serverFd);
			close(clientFd);
			exit(1);
		}
		if (pid==0) {
			/* I am the child */
			int sum;
			
			close(serverFd);
			/* receive the operands from the client */
			if (recv(clientFd,dataBuffer,sizeof(dataBuffer),0)==-1) {
				perror("recv");
			}
			sscanf(dataBuffer,"%d %d",&operand1,&operand2);
			fprintf(stderr,"server received operand1:%d operand2:%d...\n",operand1,operand2);
			sum=operand1+operand2;
			sprintf(dataBuffer,"%d",sum);
			/* send the sum to the client */
			if (send(clientFd,dataBuffer,strlen(dataBuffer)+1,0)<0) {
				perror("send");
			}
			fprintf(stderr,"server sent sum:%d...\n",sum);
			exit(0);
		}
		else {
			/* I am the parent */
			close(clientFd);
			/* wait for death of children */
			/* while (waitpid(-1,NULL,WNOHANG)>0); */
			wait(NULL);
		}
	}
}
